//********************************************************************
//  GasStation.java       Author: Ryo (Rio) Akasaka
//  
//  Last modified: Wed Feb 6, 2007 5:52
//
//  Description: A discrete clock-driven gas station, with two pumps. 
//  Customers need gas at random intervals, but will not enter the 
//  station if a queue of length 2 or greater is present at any one
//  pump. Servicing, entering the gas station, and times for queueing 
//  are assumed to occur instantaneously. A discrete clock-drive
//
//********************************************************************

import java.util.Scanner;
import java.util.Random;
import java.io.*;

public class GasStation
{ 
   public static void main(String[] args) throws IOException
  {
    Object[] pumps;
    Object[] queues;
    int servicetime = 0, maxclock = 0, numpumps = 0;
    String verbose;
    int clock = 0, carnum = 0, totaltime = 0;
    Customer car;
    int i, x;
    Reporter supervisor = new Reporter();
    
    Scanner value = new Scanner (System.in);
    InputStreamReader tempReader=new InputStreamReader(System.in);
    BufferedReader reader=new BufferedReader(tempReader);
    
    numpumps = 2; // Declares number of pumps. This should be easily changeable to n-pumps.
    pumps = new Object [numpumps];
    queues = new Object [numpumps];
    for (i=0; i< numpumps; i++) {
      Pump pumpstation = new Pump();
      pumps[i] = pumpstation;
      Queue line = new Queue(3);
      queues[i] = line;
    }
    System.out.print ("Enter the number of minutes to simulate: ");
    maxclock = value.nextInt(); 
    System.out.print ("Enter the probablility that a passing car wants gas in any given minute (0.0-1.0): "); 
    double probability = value.nextDouble();
    System.out.print ("Do you want verbose output? (y or n) Type t for total verbose: ");
    verbose = reader.readLine();
    
    Queue report = new Queue(maxclock); // Creates report queue that Supervisor will later analyze.
    
    while (clock < maxclock)
      {
	clock++; // Tick clock
	if(verbose.equalsIgnoreCase("t")) System.out.print("\n" + clock + " minute(s)\n");
	
	for (x=0; x< numpumps; x++) {
	  if (((Pump) pumps[x]).occupied(((Queue) queues[x]))) 
	    {
	      for (i=1; i< ((Queue) queues[x]).size(); i++)
		{
		  ((Customer) ((Queue) queues[x]).getnum(i)).inqueue();
		}
	      ((Pump) pumps[x]).servicing(((Queue) queues[x]), clock, x, report, verbose);
	    }
	}
	double cararrival = Math.random(); // Determine if car arrives or not
	if (cararrival < probability)
	  {
	    carnum++;
	    if(verbose.equalsIgnoreCase("t")) System.out.print(" Car " + carnum + " arrives: ");
	    car = new Customer(carnum, clock); // Create new car
	    
	    // Ensure that queue size in either pump is less than 3 (2 customers in line, 1 at the pump)
	    if (((Queue) queues[0]).size() < 3 || ((Queue) queues[1]).size() < 3)
	      {
		// Generate service time
		Random generator = new Random ();
		servicetime = generator.nextInt(7) + 4;
		car.setservice(servicetime);
		car.servtime(clock);
		if(verbose.equalsIgnoreCase("t")) System.out.println(" Pump 1 has " + ((Queue) queues[0]).size() + " customers and Pump 2 has " + ((Queue) queues[1]).size());
		if (((Queue) queues[0]).size() < ((Queue) queues[1]).size())
		  {		
		    if(verbose.equalsIgnoreCase("t")) System.out.println(" Customer chooses Pump 1");
		    ((Queue) queues[0]).enqueue(car);
		  }
		else {
		  if(verbose.equalsIgnoreCase("t")) System.out.println(" Customer chooses Pump 2");
		  ((Queue) queues[1]).enqueue(car);		
		}
	      }
	    else { 
	      if(verbose.equalsIgnoreCase("t")) System.out.println(" Customer does not enter.");
	      report.enqueue(car);
	    }
	  }
      }
    // Supervisor's report
    supervisor.information(report, verbose, probability, maxclock);
  }	
}
